# frozen_string_literal: true

module Hermes
  # Dossier de candidature , gere le listing des pieces a fournir et le statut du dossier
  class HousingRequestFolder
    include Hermes::Concerns::CommonFieldsConcern
    
    field :status, type: String # status du dossier, peut être égal à:
    #   'incomplete' || 'incomplete with validation errors' || 'validation errors' 'valid'
    field :validity_limit, type: Date # date a partir de laquelle le dossier ne sera plus valide
    field :completion, type: Integer # pourcentage de completion du dossier
    field :file_upload_from_sne_complete, type: Boolean
    belongs_to :housing_request, class_name: 'Hermes::HousingRequest'
    has_and_belongs_to_many :required_documents, class_name: 'Hermes::RequiredDocument', inverse_of: nil
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    # METODES DE CREATION DES DOCUENTS
    def compute_required_documents
      self.required_document_ids = []
      find_or_create_documents
    end
    
    def find_or_create_documents
      all_applicant_ids = [housing_request.applicant_ids, housing_request.main_applicant_id].flatten.uniq.compact
      # required_documents_common_type_list = ['hermes_identite', 'hermes_revenu_fiscal', 'hermes_ressources_mensuelles']
      all_applicant_ids.each do |applicant_id|
        compute_identity_documents(applicant_id, applicant_id ==housing_request.main_applicant_id)
        compute_ressources_documents(applicant_id, applicant_id ==housing_request.main_applicant_id)
      end
      compute_others_documents(housing_request.main_applicant, true)
      compute_family_situation_documents(housing_request.main_applicant, true)
      compute_actual_housing_documents(housing_request.main_applicant, true)
      compute_motivation_documents(housing_request.main_applicant_id.to_s, true)
    end

    def compute_identity_documents(applicant_id, is_main_applicant)

      applicant = Applicant.find(applicant_id)
      message = case applicant.nationality
                when '1'
                  'Carte d\'identité ou passeport'
                when '3'
                  'Titre de séjour ainsi que le récépissé de renouvellement de titre de séjour si celui ci est expiré'
                when '2'
                  'Carte d\'identité ou passeport du pays d\'origine'
                end
      find_or_create_required_document(message, applicant.id.to_s, is_main_applicant, true, "SPI") if message
    end


    def compute_ressources_documents(applicant_id, is_main_applicant)
      applicant = Applicant.find(applicant_id)
      rules = [
        {
          monthly_incomes: %w[AUT],
          required_documents: ['Justificatif(s) autres ressources perçues'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[BE],
          required_documents: ['carte étudiant', 'Avis d\'attribution de bourse'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[RET],
          required_documents: ['Notification de pension de retraire'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[PINV],
          required_documents: ['Notification de pension d\'invalidité'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[APP],
          required_documents: ['Contrat de travail'],
          r_for_consultation: true,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[AUT],
          required_documents: ['Toute pièce établissant la situation indiquée'],
          r_for_consultation: true,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
          work_contract_type: %w[CDI CDD],
          required_documents: ['Trois derniers bulletins de salaire ou Attestation de l\'employeur de moins de 3 mois'],
          r_for_consultation: true,
          r_for_application: true
        }, {
          monthly_incomes: %w[SAL],
         work_contract_type: %w[ART],
         required_documents: ['Dernier bilan ou attestation du comptable de l\'entreprise évaluant le salaire mensuel perçu ou tout document comptable habituellement fourni à l\'administration permettant de justifier les revenus'],
          r_for_consultation: true,
          r_for_application: true
         }, {
          monthly_incomes: %w[AMV],
          required_documents: ['Notification de pension de minimum vieillesse'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[PAV PAR],
          required_documents: ['Extrait de jugement ou autre document démontrant le versement de la pension'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: %w[CHO],
          required_documents: ['Avis de paiement d\'allocation d\'aide de retour à l\'emploi'],
          r_for_consultation: false,
          r_for_application: true
        }, {
          monthly_incomes: [
            'AF',
            'AAH',
            'AEEH',
            'AJPP',
            'RSA',
            'PAJE',
            'PDA'
          ],
          required_documents: ['Attestation de la caisse d\'allocation familiales (CAF) ou de la Mutualité sociale agricole (MSA)'],
          r_for_consultation: false,
          r_for_application: true
        }
      ]
      rules.each do |rule|
        work_contract_type_condition = true
        if rule[:work_contract_type]
          work_contract_type_condition = rule[:work_contract_type].include? applicant.work_contract_type
        end
        is_doc_needed = false
        rule[:monthly_incomes].each do |ressource_label|
          next unless applicant.monthly_incomes &&
              (
              applicant.monthly_incomes[ressource_label] ||
                  applicant.monthly_incomes[ressource_label.to_sym] ||
                  0
              ) > 0 && work_contract_type_condition
          
          is_doc_needed = true
        end
        next unless is_doc_needed
        
        rule[:required_documents].each do |req_doc|
          find_or_create_required_document(req_doc, applicant.id.to_s, rule[:r_for_consultation], rule[:r_for_application],"SPR" )
        end
      end
    end


    def compute_others_documents(applicant, is_main_applicant)
      return unless applicant.family_situation == 'V'
      
      find_or_create_required_document(

        'Certificat de décès ou livret de famille',
        applicant.id.to_s,
        true,
        true,
        "SPR"
      )
      
      #todo rajouter le dernier document quant on aura les specs
    end

    def compute_family_situation_documents(applicant, is_main_applicant)
      rules = [
        {
          situation: 'P',
          required_document: 'Attestation d\'enregistrement de PACS'
        }, {
          situation: 'M',
          required_document: 'Livret de famille ou tout document équivalant démontrant le mariage'
        }, {
          situation: 'D',
          required_document: 'Extrait de jugement de divorce ou  extrait de la convention homologué en cas de divorce par consentement mutuel'
        }, {
            situation: 'S',
            required_document: "(Vous êtiez marié(e))
« Ordonnance de non conciliation ou acte de saisine du juge aux affaires familiales ou justificatif de l'avocat du demandeur en cas de divorce par consentement mutuel »
ou
(Vous êtiez pacé(e))
« Mention de la dissolution dans l'acte de naissance ou Décision du juge prise en application de l'article 257 du code civil ou ordonnance de protection délivré par le juge aux affaires familiales ou Récépissé d'enregistrement de la déclaration de séparation »
ou
(Autre)
« Récépissé du dépôt de plainte si violence au sein du couple »
"
        }

      ]
      
      rules.each do |rule|
        find_or_create_required_document(rule[:required_document], applicant.id.to_s, false, true, "SF") if rule[:situation] == applicant.family_situation
      end
    end


    def compute_actual_housing_documents(applicant, is_main_applicant)
      rules = [
        {
          actual_housing_detail: %w[POCC],
          required_documents: ['Acte de propriété', 'Plan de financement']
        }, {
          actual_housing_detail: %w[RSLL RHVS HSH HCDFCM],
          required_documents: ['Attestation de la structure d\'hébergement ou du gestionnaire du logement-foyer']
        }, {
          actual_housing_detail: %w[FONC],
          required_documents: ['Contrat de bail ou dernière quittance de loyer ou tout document attestant de la situation']
        }, {
          actual_housing_detail: %w[HCPE HCP],
          required_documents: ['Attestation de la personne qui héberge']
        }, {
          actual_housing_detail: %w[SD CAMCAR HHOT HABMOB],
          required_documents: ['Attestation d\'un travailleur social, d\'une association ou certificat de domiciliation ou autre document démontrant l\'absence de logement']
        }, {
          actual_housing_detail: %w[HLM LP],
          required_documents: ["Dernière quittance de loyer et Contrat de bail
ou Attestation du bailleur indiquant que le locataire est à jour du paiement des loyers et charges"]
        },{
          actual_housing_detail: %w[SLHT],
          required_documents: ["(Vous êtes sous locataire)
(« Bail et dernière quittance de loyer ou attestation du bailleur indiquant que le locataire est à jour des loyers et charges ou tout moyen de preuve des paiements effectués. »

(Vous êtes hébergé dans un logement à titre temporaire)
« Attestation de la personne qui héberge »
"]
        }

      ]
      
      rules.each do |rule|
        next unless applicant.actual_housing_details &&
            rule[:actual_housing_detail].include?(applicant.actual_housing_details['actual_housing_type'])
        
        rule[:required_documents].each do |req_doc|
          find_or_create_required_document(req_doc, applicant.id.to_s, false, true, "SPL")
        end
      end
    end



    def compute_motivation_documents(applicant_id, is_main_applicant)
      all_motivations = (1..3).map { |i| housing_request.send("motivation#{i}") }
      rules = [
        {
          motivation: '08',
          required_document: 'Certificat médical'
        }, {
          motivation: '28',
          required_document: 'Attestation de l\'employeur actuel ou futur justifiant votre mobilité professionnelle'
        }, {
          motivation: '15',
          required_document: 'Attestation de dépôt de demande de regroupement familial'
        }, {
          motivation: '02',
          required_document: 'Toute pièce établissant de la démolition de votre logement actuel'
        }, {
          motivation: '07',
          required_document: 'Carte d\'invalidité ou décision d\'une commission administrative compétente (MDPH...)'
        }, {
          motivation: '05',
          required_document: 'Commandement de payer ou assignation à comparaître ou jugement prononçant l\'expulsion ou commandement de quitter les lieux'
        }, {
          motivation: '22',
          required_document: 'Plan d\'apurement de la dette ou démarches en cours attestées par un travailleur social ou une association ou autre document démontrant les difficultés'
        }, {
          motivation: '26',
          required_document: 'Agrément assistant(e) maternel(le)'
        }, {
          motivation: '06',
          required_document: 'Décision du juge prise en application de l\'article 257 du code civil attestant de la situation d\'urgence ou ordonnance de protection délivrée par le juge aux affaires familiales ou récépissé de dépôt de plainte'
        }, {
          motivation: '04',
          required_document: 'Lettre de congé du propriétaire ou jugement prononçant la résiliation du bail '
        },{
          motivation: '24',
          required_document: "Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement ou copie du jugement d'un tribunal statuant sur l'indécence du logement ou attestation de la CAF/MSA ou autre document démontrant l'indécence du logement"
        }, {
          motivation: '25',
          required_document: "« Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, attestation de la CAF ou de la CMSA, copie d'une décision de l'administration (mise en demeure, arrêté préfectoral ou municipal d'insalubrité remédiable ou irrémédiable ou de péril, ou de fermeture administrative s'il s'agit d'un hôtel meublé) ou autre document démontrant l'insalubrité ou la dangerosité du logement ou de l'immeuble »

(Logement impropre à l'habitation )
« Document établi par un service public, un travailleur social, un professionnel du bâtiment ou une association ayant pour objet l'insertion ou le logement, photos, jugement du tribunal, arrêté préfectoral de mise en demeure de faire cesser l'occupation des lieux ou autre document démontrant le caractère impropre à l'habitation »
"
        }

      ]
      
      rules.each do |rule|
        find_or_create_required_document(rule[:required_document], applicant_id, false, true, "MD") if all_motivations.include?(rule[:motivation])
      end
    end
    
    # methode permettant d'associer au housing_request_folder un requiredDocument
    def find_or_create_required_document(type, applicant_id, is_required_for_consultation, is_required_for_application, category  )
      required_document = Hermes::RequiredDocument.find_by(
          type: type,
          applicant_id: applicant_id.to_s
      )

      if required_document
        if is_required_for_consultation
          required_document.folder_where_is_required_for_consultation = [id] unless required_document.folder_where_is_required_for_consultation
          required_document.folder_where_is_required_for_consultation.push(id) unless required_document.folder_where_is_required_for_consultation.include?(id)
        end
        if is_required_for_application
          required_document.folder_where_is_required_for_application = [id] unless required_document.folder_where_is_required_for_application
          required_document.folder_where_is_required_for_application.push(id) unless required_document.folder_where_is_required_for_application.include?(id)
        end
        required_document.save #todo: OPTIMISATION ne pas save si pas de changement
      end
      required_document_ids.push(
        if !required_document
          Hermes::RequiredDocument.create(
            type: type,
            applicant_id: applicant_id.to_s,
            status: 'Piece manquante',
            folder_where_is_required_for_consultation: is_required_for_consultation ? [id] : [],
            folder_where_is_required_for_application: is_required_for_application ? [id] : [],
            category: category
          ).id
        else
          required_document.id
        end
      )
    end
    
    def get_file_list(unique_number)
      so = Hermes::BoClient::ListingPJTask.new(
          {
              housing_request_id: housing_request.id,
              num_unique: unique_number
          }
      )
      puts so.to_xml
      ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
    end
    
    def get_files_from_sne(unique_number)
      housing_request_unique_numbers = housing_request.nuds.only(:unique_number).pluck(:unique_number)
     # return unless unique_number && (housing_request_unique_numbers.include? unique_number )
    #  file_list = get_file_list(unique_number)
      self.file_upload_from_sne_complete = true
    end
  end
end
