require 'rails_helper'

RSpec.describe(Hermes::Applicant, type: :model) do
  it 'has a valid factory' do
    expect(FactoryBot.create(:applicant)).to be_valid
  end

  let!(:user1) { create(:user) }

  # describe 'validate title presence' do
  #  it 'is invalid without a first_name' do
  #    expect(FactoryBot.build(:applicant, first_name: nil)).not_to be_valid
  #  end
  # end

  describe 'relations' do
    let!(:entity1) { create(:entity, siret: 'test1') }
    let!(:applicant) { create(:applicant, main_employer_siret: 'test1', user_id: user1.id) }
    let!(:applicant2) { create(:applicant) }
    let!(:housing_request) { create(:housing_request, main_applicant: applicant, is_active: true) }

    it 'put referent' do
      applicant_link = Hermes::ApplicantLink.create(type: 'link', applicant_ids: [applicant, applicant2].map { |elt| elt.id.to_s }, housing_request: housing_request)
      expect(Hermes::ApplicantLink.where(applicant_ids: applicant.id.to_s).count).to eq(1)
      expect(applicant.reload.links.count).to eq(1)
      expect(applicant.link_types_for(applicant2.id)).to eq(['link'])
      expect(applicant.entity.id).to eq entity1.id
      expect(entity1.group.user_ids).to include applicant.user_id
      expect(applicant.reload.readers).to include "g_#{entity1.supervisor_group.id}"
      expect(applicant.reload.writers).to include "g_#{entity1.manager_group.id}"
      expect(housing_request.readers).to eq applicant.readers
      expect(housing_request.writers).to eq applicant.writers
    end

    describe 'changing applicant main_employer_siret' do
      let!(:entity2) { create(:entity, siret: 'test2') }

      before :each do
        applicant.main_employer_siret = 'test2'
        applicant.save
      end

      it 'should has changed entity' do
        r_applicant = applicant.reload
        r_housing_request = housing_request.reload
        expect(r_applicant.entity.id).to eq entity2.id
        expect(entity1.group.user_ids).to_not include r_applicant.user_id
        expect(r_applicant.readers).not_to include "g_#{entity1.supervisor_group.id}"
        expect(r_applicant.writers).not_to include "g_#{entity1.manager_group.id}"
        expect(entity2.group.user_ids).to include r_applicant.user_id
        expect(r_applicant.readers).to include "g_#{entity2.supervisor_group.id}"
        expect(r_applicant.writers).to include "g_#{entity2.manager_group.id}"
        expect(r_housing_request.readers).to eq applicant.readers
        expect(r_housing_request.writers).to eq applicant.writers
      end
    end
  end

  # BEFORE ACTION UNIT TESTS
  context 'before action unit test context' do
    # WHERE : we create an applicant with an user and a first active request
    let!(:user) { create(:user, first_name: 'Michel', last_name: 'Dupont', email: 'michel.dupont@gmail.com') }
    let!(:user2) { create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com') }
    let!(:entity) { create(:entity, {title: 'title1', siret: '123456'}) }
    let!(:applicant) { create(:applicant, user: user, entity: entity, status: 'OK', monthly_incomes: {drug_selling: 20}, monthly_charges: {dog_food: 5}) }
    let!(:applicant2) { create(:applicant, user: user2, entity: entity, status: 'OK', monthly_incomes: {porn: 30}, monthly_charges: {cat_food: 10}) }
    let!(:housing_request) { create(:housing_request, main_applicant: applicant, coapplicants: [{rel_type: 'marié', id: applicant2.id.to_s}], dependents: [{rel_type: 'enfant à charge', monthly_incomes: {pokemon_selling: 5}, monthly_charges: {unicorn_food: 10}}], is_active: true) }

    # inherit_user_datas
    describe 'inherit_user_datas unit test' do
      let!(:user2) { create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com') }
      before :each do
        # WHEN : we change user and save
        applicant.main_user_id = user2.id
        applicant.save
      end
      # THEN : we werify the users data has changed
      it 'applicant user data has changed and first_active_request.name_has_changed == true' do
        expect(applicant.first_name).to eq('Martin')
        expect(applicant.last_name).to eq('Durand')
        expect(applicant.full_name).to eq('Martin Durand')
        expect(applicant.email).to eq('martin.durand@gmail.com')

        reloaded_hr = housing_request.reload
        expect(reloaded_hr.requester_firstname).to eq('Martin')
        expect(reloaded_hr.requester_lastname).to eq('Durand')
        expect(reloaded_hr.requester_fullname).to eq('Martin Durand')
        expect(reloaded_hr.requester_email).to eq('martin.durand@gmail.com')
      end
    end

    describe 'not_inherit_user_datas unit test' do
      let!(:applicant3) do
        create(
          :applicant,
          first_name: 'Jackie',
          last_name: 'Mike',
          full_name: 'Jackie Mike',
          email: 'jackie.mike@gmail.com',
          entity: entity,
          status: 'OK',
          monthly_incomes: { porn: 30 },
          monthly_charges: { cat_food: 10 }
        )
      end

      before :each do
        # WHEN : we change user and save
        applicant3.first_name = 'Martin'
        applicant3.last_name = 'Durand'
        applicant3.full_name = 'Martin Durand'
        applicant3.email = 'martin.durand@gmail.com'
        applicant3.save
      end

      # THEN : we werify the users data has changed
      it 'applicant user data has changed and first_active_request.name_has_changed == true' do
        expect(applicant3.first_name).to eq('Martin')
        expect(applicant3.last_name).to eq('Durand')
        expect(applicant3.full_name).to eq('Martin Durand')
        expect(applicant3.email).to eq('martin.durand@gmail.com')
      end
    end

    describe 'inherit_user_datas unit test' do
      identity = {
        first_name: 'Martin',
        last_name: 'Durand',
        email: 'martin.durand@gmail.com',
        birth_date: DateTime.new(1918, 12, 12)
      }
      let!(:applicant3) do
        create(
          :applicant,
          first_name: 'julien',
          last_name: 'papaouto',
          email: 'papaouto@gmail.com',
          birth_date: DateTime.new(2008, 12, 12)
        )
      end
      let!(:housing_request3) { create(:housing_request, main_applicant_id: applicant3.id.to_s, is_active: true) }
      identity.each do |key, val|
        describe 'change in applicant ' + key.to_s do
          before :each do
            # WHEN : we change user and save
            applicant3.reload
            applicant3[key] = val

            applicant3.save
          end

          it ' is repercuted in the housing_request' do
            r_applicant3 = applicant3.reload
            reloaded_hr3 = housing_request3.reload
            expect(applicant3[key]).to eq(val)
            case key
            when :first_name
              expect(r_applicant3.first_name).to eq(val)
              expect(reloaded_hr3.requester_firstname).to eq(val)
            when :last_name
              expect(r_applicant3.last_name).to eq(val)
              expect(reloaded_hr3.requester_lastname).to eq(val)
            when :email
              expect(r_applicant3.email).to eq(val)
              expect(reloaded_hr3.requester_email).to eq(val)
            when :birth_date
              expect(r_applicant3.birth_date).to eq(val)
              expect(reloaded_hr3.requester_birth_date).to eq(val)
            end
          end
        end
      end
    end

    #update_employer_data
    describe 'update_employer_data unit test' do
      let!(:entity2) {create(:entity, {title: 'title2', siret: '7890'})}
      before :each do
        # WHEN : entity_id_changed?
        applicant.entity = entity2
        applicant.save
      end
      #TEHN : we werify the users data has changed
      it 'employer data has changed' do
        expect(applicant.main_employer_name).to eq('title2')
        expect(applicant.main_employer_siret).to eq('7890')
      end
    end

    #update_birth_dependencies
    describe 'update_birth_dependencies' do
      before :each do
        # WHEN : birth date_changed?
        applicant.birth_date = Date.new(1918, 12, 12)
        applicant.save
      end
      #TEHN : we verify the users data has changed
      it 'set the birth_plus_30 correctly' do
        expect(applicant.birth_plus_30.to_datetime).to eq((Date.new(1918, 12, 12) + 30.years).to_datetime)
      end
    end

    #update_status_dependencies
    describe 'update_status_dependencies' do
      before :each do
        applicant.status_updated_at = (Time.current - 1.day).to_datetime
        applicant.save
        # WHEN : status_changed?
        applicant.status = 'KO'
        applicant.save
      end
      #TEHN : we verify the users data has changed
      it 'set the status_updated_at to now' do
        expect(applicant.status_updated_at).to be > (Time.now - 1.minute).to_datetime
      end
    end

    #check_all
    describe 'check_all' do
      #check_monthly_revenues_dependency
      describe 'check_monthly_revenues_dependency' do
        describe 'change in monthly_incomes' do
          before :each do
            applicant.monthly_incomes = { drug_selling: 10 }
            applicant.save
          end
          it 'has made changes in first_active_request' do
            reloaded_hr = housing_request.reload
            expect(reloaded_hr.monthly_revenues).to eq(20)
          end
        end
        describe 'change in monthly_charges' do
          before :each do
            applicant.monthly_charges = {dog_food: 0} # ON MET KOI A DEDANS???
            applicant.save
          end

          it 'has made changes in first_active_request' do
            reloaded_hr = housing_request.reload
            expect(reloaded_hr.monthly_revenues).to eq(35)
          end
        end
      end
      #check_category_dependency
      describe 'check_category_dependency' do
        before :each do
          applicant.family_situation = 'relation libre polyamoureuse'
        end
        #we test if should_update_eligibility before save
        it 'self.should_update_eligibility = true' do
          applicant.before_actions
          expect(applicant.should_update_eligibility).to eq(true)
          expect(applicant.first_active_request.category_may_change).to eq(true)
          applicant.save
          expect(housing_request.reload.category_may_change).to eq(false)
          expect(applicant.reload.should_update_eligibility).to eq(nil)
        end
      end

      #check_eligibility_dependency
      describe 'check_eligibility_dependency' do
        %w[
          entity_id
          mutation
          mobility
          priority_public
          single_parent
        ].concat((1..12).map {|i| "treshold_category_#{i}"}).each do |changed_field|
          describe 'when changing ' + changed_field.to_s + 'on applicant'
          before :each do
            applicant[changed_field] = !applicant[changed_field]
          end
          it 'has trigger housing_request check' do
            applicant.before_actions
            expect(applicant.should_update_eligibility).to eq(true)
            expect(applicant.first_active_request.category_may_change).to eq(true)
            applicant.save
            expect(housing_request.reload.category_may_change).to eq(false)
            expect(applicant.reload.should_update_eligibility).to eq(nil)
          end
        end
      end
    end

    #check_year_revenues
    describe 'check_year revenue dependency' do
      before :each do
        applicant.amount_year_minus_two = 20000
      end
      #we test if should_update_eligibility before save
      it 'self.should_update_eligibility = true' do
        applicant.before_actions
        expect(applicant.should_update_eligibility).to eq(true)
        expect(applicant.first_active_request.tax_revenues_will_change).to eq(true)
        applicant.save
        expect(housing_request.reload.tax_revenues_will_change).to eq(false)
        expect(applicant.reload.should_update_eligibility).to eq(nil)
      end
    end
  end
end
